#' Data generation for simulation experimentation.
#'
#' This function uses the predictor covariance matrix, marginal predictor characteristics, and estimated relationships between predictors and potential outcomes from an ECLSK data set to simulate data.
#' @param n The total sample size for the data set. The default value is \code{7362}.
#' @keywords Data Generation Model ECLSK
#' @export

dgpECLSK <- function(n = 7362) {
  # Sample var/cov matrix
  covv <- rfvarsel::cov_mat
  corr <- cov2cor(covv)
  # Range violation! Corr[ 8 , 9 ] must be between -0.112 and 0.54
  corr[8,9] <- corr[9,8] <- -.112
  # Generate sigma.star matrix for BinNor joint data generation
  sig_star <- BinNor::compute.sigma.star(no.bin = length(rfvarsel::props),
                                         no.nor = length(rfvarsel::mns),
                                         prop.vec.bin = rfvarsel::props,
                                         corr.vec = NULL,
                                         corr.mat = corr)
  # The resulting correlation matrix is not positive definite so nearest
  # positive definite matrix will be used.
  dat <- BinNor::jointly.generate.binary.normal(
    no.rows = n,
    no.bin = length(rfvarsel::props),
    no.nor = dim(corr)[1] - length(rfvarsel::props),
    prop.vec.bin = rfvarsel::props,
    mean.vec.nor = rfvarsel::mns,
    var.nor = rfvarsel::vars,
    sigma_star = sig_star$sigma_star)
  dat <- data.frame(dat)
  names(dat) <- rfvarsel::nms
  Y0 <- predict(object = rfvarsel::lm03, newdata = dat[which(dat$F5SPECS == 0),])
  Y1 <- predict(object = rfvarsel::lm13, newdata = dat[which(dat$F5SPECS == 1),])
  dat$Y <- 0
  dat[which(dat$F5SPECS == 0), "Y"] <- Y0
  dat[which(dat$F5SPECS == 1), "Y"] <- Y1
  foo <- function(vec) {length(table(vec))}
  dichs <- which(sapply(X = dat, FUN = foo) == 2)
  dat[,dichs] <- lapply(X = dat[,dichs], FUN = as.factor)
  dat
}

#' Data generation for simulation experimentation.
#'
#' This function is similar to \code{dgpLINEAR} with the exception that variables X2 and X3 are related to exposure and potential outcomes only through their interaction.
#' @param n The total sample size for the data set. The intercept of the propensity score model is set to 0 so that the groups sizes are approximately equal. The default value is 500.
#' @param noise The number of additional noise variables to be simulated. Noise variables are variables that do not have a causal link to either exposure or potential outcomes.
#' @keywords Data Generation Model Quadratic
#' @export

dgpINT <- function(n = 500, noise = 12) {
  sig <- diag(8)
  dich <- function(vec) { vec >= mean(vec) }
  X <- mvtnorm::rmvnorm(n = n, sigma = sig)
  X[,1] <- dich(X[,1]); X[,6] <- dich(X[,6]); X[,7] <- dich(X[,7])
  coefs_ps <- t(t(c(log(2), 0, 0, log(2), 0, log(2), 0, 0)))
  coefs_oc <- t(t(c(1, 0, 0, 0, 1, 0, 2, 0)))
  ps <- (1 + exp( -(0 + X %*% coefs_ps + 1*X[,2]*X[,3]) ) )^-1
  T <- rbinom(n = n, size = 1, prob = ps)
  Y <- 0 + X %*% coefs_oc + 1*X[,2]*X[,3] + 0.5*T + rnorm(n, sd = 1)
  dch <- floor(noise/2)
  Xaug <- mvtnorm::rmvnorm(n = n, sigma = diag(noise))
  Xaug[,1:dch] <- apply(Xaug[,1:dch], 2, dich)
  dat <- data.frame(cbind(X, Xaug, Y, T))
  dat$T <- factor(dat$T)
  names(dat) <- c(paste0("X", 1:(8 + noise)), "Y", "T")
  dat
}

#' Data generation for simulation experimentation.
#'
#' This function generates continuous predictor variables such that X1, X2, X3, X4, and X6 have a direct effect on exposure T and X1, X2, X3, X5, and X7 havea a direct effect on potential outcomes.
#' @param n The total sample size for the data set. The intercept of the propensity score model is set to 0 so that the groups sizes are approximately equal. The default value is 500.
#' @param noise The number of additional noise variables to be simulated. Noise variables are variables that do not have a causal link to either exposure or potential outcomes.
#' @keywords Data Generation Model Quadratic
#' @export

dgpLINEAR <- function(n = 500, noise = 12) {
  sig <- diag(8)
  dich <- function(vec) { vec >= mean(vec) }
  X <- mvtnorm::rmvnorm(n = n, sigma = sig)
  X[,1] <- dich(X[,1]); X[,6] <- dich(X[,6]); X[,7] <- dich(X[,7])
  coefs_ps <- t(t(c(log(2), log(2), log(2), log(2), 0, log(2), 0, 0)))
  coefs_oc <- t(t(c(1, 1, 1, 0, 1, 0, 2, 0)))
  ps <- (1 + exp( -(0 + X %*% coefs_ps) ) )^-1
  T <- rbinom(n = n, size = 1, prob = ps)
  Y <- 0 + X %*% coefs_oc + 0.5*T + rnorm(n, sd = 1)
  dch <- floor(noise/2)
  Xaug <- mvtnorm::rmvnorm(n = n, sigma = diag(noise))
  Xaug[,1:dch] <- apply(Xaug[,1:dch], 2, dich)
  dat <- data.frame(cbind(X, Xaug, Y, T))
  dat$T <- factor(dat$T)
  names(dat) <- c(paste0("X", 1:(8 + noise)), "Y", "T")
  dat
}

#' Data generation for simulation experimentation.
#'
#' This function generates data such that X1, X2, and X3 affect both the exposure T and the potential outcomes. X4 is an exposure-only predictor. X5 is an outcome-only predictor. X6 is a predictor for exposure T and X8. X7 is predictor for potential outcomes and X8. Thus, X8 is a collider on the path from T the potential outcome.
#' @param n The total sample size for the data set. The intercept of the propensity score model is set to 0 so that the groups sizes are approximately equal. The default value is 500.
#' @param noise The number of additional noise variables to be simulated. Noise variables are variables that do not have a causal link to either exposure or potential outcomes.
#' @keywords Data Generation Model
#' @export

dgpCOLLIDER <- function(n = 500, noise = 12) {
  sig <- diag(8)
  dich <- function(vec) { vec >= mean(vec) }
  X <- mvtnorm::rmvnorm(n = n, sigma = sig)
  X[,1] <- dich(X[,1]); X[,6] <- dich(X[,6]); X[,7] <- dich(X[,7])
  X[,8] <- 2*X[,6] + 2*X[,7] + rnorm(n, sd = 1)
  coefs_ps <- t(t(c(log(2), log(2), log(2), log(2), 0, log(2), 0, 0)))
  coefs_oc <- t(t(c(1, 1, 1, 0, 1, 0, 2, 0)))
  ps <- (1 + exp( -(0 + X %*% coefs_ps) ) )^-1
  T <- rbinom(n = n, size = 1, prob = ps)
  Y <- 0 + X %*% coefs_oc + 0.5*T + rnorm(n, sd = 1)
  dch <- floor(noise/2)
  Xaug <- mvtnorm::rmvnorm(n = n, sigma = diag(noise))
  Xaug[,1:dch] <- apply(Xaug[,1:dch], 2, dich)
  dat <- data.frame(cbind(X, Xaug, Y, T))
  dat$T <- factor(dat$T)
  names(dat) <- c(paste0("X", 1:(8 + noise)), "Y", "T")
  dat
}

#' Data generation for simulation experimentation.
#'
#' This function generates data such that X1, X2, and X3 affect both the exposure T and the potential outcomes. X4 is an exposure-only predictor. X5 is an outcome-only predictor. X6 is a predictor for exposure T and X8. X7 is predictor for potential outcomes and X8. Thus, X8 is a collider on the path from T the potential outcome.
#' @param n The total sample size for the data set. The intercept of the propensity score model is set to 0 so that the groups sizes are approximately equal. The default value is 500.
#' @param noise The number of additional noise variables to be simulated. Noise variables are variables that do not have a causal link to either exposure or potential outcomes.
#' @keywords Data Generation Model
#' @export

dgpINTCOLL <- function(n = 500, noise = 12) {
  sig <- diag(8)
  dich <- function(vec) { vec >= mean(vec) }
  X <- mvtnorm::rmvnorm(n = n, sigma = sig)
  X[,1] <- dich(X[,1]); X[,6] <- dich(X[,6]); X[,7] <- dich(X[,7])
  X[,8] <- 2*X[,6] + 2*X[,7] + rnorm(n, sd = 1)
  coefs_ps <- t(t(c(log(2), 0, 0, log(2), 0, log(2), 0, 0)))
  coefs_oc <- t(t(c(1, 0, 0, 0, 1, 0, 2, 0)))
  ps <- (1 + exp( -(0 + X %*% coefs_ps + 2*X[,2]*X[,3]) ) )^-1
  T <- rbinom(n = n, size = 1, prob = ps)
  Y <- 0 + X %*% coefs_oc + 2*X[,2]*X[,3] + 0.5*T + rnorm(n, sd = 1)
  dch <- floor(noise/2)
  Xaug <- mvtnorm::rmvnorm(n = n, sigma = diag(noise))
  Xaug[,1:dch] <- apply(Xaug[,1:dch], 2, dich)
  dat <- data.frame(cbind(X, Xaug, Y, T))
  dat$T <- factor(dat$T)
  names(dat) <- c(paste0("X", 1:(8 + noise)), "Y", "T")
  dat
}


