#' Variable selection based on permutation of conditional random forest permutation variable importance
#'
#' This function implements variable selection based on conditional importance values from random forest runs.
#' @param x A matrix of predictors.
#' @param y The outcome variable.
#' @param perms The number of permutations used to create a null distribtuion for importance values. The default value is \code{10}.
#' @param ntree Number of trees for the random forest run. The default value is \code{50}.
#' @param pctl The percentile of the null distribution of importance used as a threshold for variable selection. The default is \code{pctl = .95}.
#' @param scale Should the permutation importances be ``scaled''? The default value is \code{FALSE}.
#' @param permcores Number of processor cores to use for parallel computing with package \pkg{parallel} for running the \code{perms} permutations. The default is no parallel computing (i.e., \code{permcores = 1}). Note that pkg{parallel} is not compatible with Windows versions of R. Specifically, if running Windows, \code{mclapply} will revert to \code{lapply}, so the function will still run, but with no parallelization.
#' @param plot Should a plot of variable importance and permutation thresholds be produced? The default value is \code{FALSE}.
#' @keywords Random Forest Variable Selection via Permutation of Conditional Permutation Importance
#' @export

RFVS_perm_cond <- function(x, y, perms = 10,
                       ntree = 50,
                       pctl = .95,
                       scale = FALSE,
                       plot = FALSE,
                       permcores = 1) {
  if(is.null(x) | length(x) == 0) return(NULL)
  if(dim(x)[2] == 0) return(NULL)
  if(sum(is.na(x), is.na(y)) > 0) stop("No missing values are allowed.")
  n <- dim(x)[1]
  p <- dim(x)[2]
  if(length(p) == 0) return(NULL)
  perm_dat_list <- vector(mode = "list", length = (perms + 1))
  perm_dat_list[[1]] <- x
  perm_dat <- function(dat) { pdat <- dat[sample(x = 1:dim(dat)[1], replace = FALSE), , drop = FALSE] }
  perm_dat_list[2:length(perm_dat_list)] <- replicate(n = perms, expr = perm_dat(x), simplify = FALSE)
  imps <- parallel::mclapply(X = perm_dat_list, FUN = CRF_vimp, y = y, ntree = ntree, scale = scale, mc.cores = permcores)
  imp <- imps[[1]]
  imps <- matrix(unlist(imps[2:length(perm_dat_list)]), nrow = length(imps[[2]]), ncol = perms, byrow = FALSE)
  pctls <- apply(X = imps, MARGIN = 1, FUN = quantile, probs = pctl)
  ord <- order(imp, decreasing = TRUE)
  imp_ord <- imp[ord]; pctls_ord <- pctls[ord]
  if(plot) {
    par(mar = c(5.1, 6.5, 1.1, 2.1))
    hts <- imp_ord[length(imp_ord):1]
    hts[which(hts < 0)] <- 0
    barplot(height = hts, horiz = TRUE,
            las = 1, xlab = "Random Forest Conditional Permutation Importance",
            main = "", col = "skyblue",
            cex.names=.8, xlim = c(0, max(c(imp,pctls))))
    barplot(height = pctls_ord[length(pctls_ord):1], horiz = TRUE,
            col = rgb(1, 0, 0, alpha=0.2),
            add = TRUE) }
  out <- cbind(imp_ord, pctls_ord)
  out <- data.frame(out)
  rownames(out) <- names(imp_ord)
  colnames(out) <- c("Imp", "Pctl")
  return(out)
}

