#' Implementation of de Luna, Waernbaum, & Richardson's (2011) "XY" algorithm. Calls \code{RFVS_perm_cond} for variable selection.
#'
#' This function implements de Luna, Waernbaum, & Richardson's (2011) "XY" algorithm using random forests with conditional variable importance under permutation for testing conditional independence.
#' @param X Matrix of predictors.
#' @param Y The outcome variable.
#' @param T The treatment (exposure) variable. Must be dichotomous 0/1.
#' @param perms The number of permutation replications used to create a null distribtuion for importance values. The default value is \code{10}.
#' @param ntree The number of trees for the random forest run. The default value is \code{50}.
#' @param pctl The percentile of the null permutation distribution of importance used as a threshold for variable selection. The default is \code{pctl = .95}.
#' @param scale Should the permutation importances be ``scaled''? The default value is \code{FALSE}.
#' @param permcores Number of processor cores to use for parallel computing with package \pkg{parallel} for running the \code{perms} permutations. The default is no parallel computing (i.e., \code{permcores = 1}). Note that pkg{parallel} is not compatible with Windows versions of R. Specifically, if running Windows, \code{mclapply} will revert to \code{lapply}, so the function will still run, but with no parallelization.
#' @keywords Random Forest Variable Selection Permutation Importance
#' @export

RFXY_cond <- function(X, Y, T,
                  perms = 10,
                  ntree = 50,
                  pctl = .95,
                  scale = FALSE,
                  permcores = 1) {
  if(is.null(X)) return(NULL)
  if(dim(X)[2] == 0) return(NULL)
  if(sum(is.na(X), is.na(Y), is.na(T)) > 0) stop("No missing values are allowed.")
  if(length(table(T)) != 2) stop("Exposure variable T must have exactly two categories.")
  T <- factor(T); levels(T) <- 0:1
  Xsubs0 <- X[T == 0, , drop = FALSE]; Xsubs1 <- X[T == 1, , drop = FALSE]
  Y0 <- Y[T == 0]; Y1 <- Y[T == 1]
  X0 <- RFVS_perm_cond(x = Xsubs0, y = Y0, ntree = ntree,
                       perms = perms, pctl = pctl,
                       scale = scale, permcores = permcores)
  X1 <- RFVS_perm_cond(x = Xsubs1, y = Y1, ntree = ntree,
                       perms = perms, pctl = pctl,
                       scale = scale, permcores = permcores)

  X0 <- rownames(X0)[which(X0[,1] > 0 & X0[,1] >= X0[,2])]
  X1 <- rownames(X1)[which(X1[,1] > 0 & X1[,1] >= X1[,2])]
  XY <- sort(union(X0, X1))
  if(is.null(XY)) return(NULL)
  if(length(XY) == 0) return(NULL)
  return(XY) }


