#' Wrapper for \code{RFXY_cond}.
#'
#' This function implements causal variable selection by combining de Luna, Waernbaum, & Richardson's (2011) "XY" algorithm with permutation-based tests of conditional independence based on conditional random forest permutation imporance. The data are preprocessed to screen out noise variables using traditional random forest permutation importance, which is much less computationally expensive. Then, a final pass is made using conditional variable importance.
#' @param X Matrix of predictors.
#' @param Y The outcome variable. Continuous outcome and dichotomous factor outcome are permitted.
#' @param T The treatment (exposure) variable. Must be dichotomous 0/1.
#' @param perms The number of permutation replications used to create a null distribution for preprocessing with traditional RF importance values. The default value is \code{20}.
#' @param ntree The number of trees for the random forest runs for traditional RF importance values. The default value is \code{200}.
#' @param permsCond The number of permutation replications used to create a null distribution for preprocessing with conditional RF importance values. The default value is \code{10}.
#' @param ntreeCond The number of trees for the random forest runs for conditional RF importance values. The default value is \code{50}.
#' @param pctl The percentile of the null permutation distribution of importance used as a threshold for variable selection. The default is \code{pctl = .95}.
#' @param scale Should the permutation importances be ``scaled''? The default value is \code{FALSE}.
#' @param sidak Should a Sidak correction for multiple tests be made based on the number of predictors? The default is \code{TRUE}.
#' @param iterate Should variable selection using traditional RF importance run iteratively until no new variables are dropped? The default is \code{FALSE}.
#' @param permcores Number of processor cores to use for parallel computing with package \pkg{parallel} for running the \code{perms} permutations. The default is no parallel computing (i.e., \code{permcores = 1}). Note that pkg{parallel} is not compatible with Windows versions of R. Specifically, if running Windows, \code{mclapply} will revert to \code{lapply}, so the function will still run, but with no parallelization.
#' @keywords Random Forest Variable Selection Conditional Permutation Importance
#' @export

RFXY_cond_wrap <- function(X, Y, T,
                           perms = 20,
                           ntree = 200,
                           permsCond = 10,
                           ntreeCond = 50,
                           pctl = .95,
                           scale = FALSE,
                           sidak = TRUE,
                           iterate = FALSE,
                           permcores = 1) {
  nms <- RFXY_trad_wrap(X = X, Y = Y, T = T,
                        perms = perms, ntree = ntree,
                        pctl = pctl, scale = scale,
                        iterate = iterate, sidak = sidak)
  out <- RFXY_cond(X = X[,nms], Y = Y, T = T,
                   perms = permsCond, ntree = ntreeCond,
                   pctl = pctl, scale = scale,
                   permcores = permcores)
  out
}


