#' Implementation of de Luna, Waernbaum, & Richardson's (2011) "XY" algorithm. Calls RFVS_perm_trad for variable selection.
#'
#' This function implements de Luna, Waernbaum, & Richardson's (2011) "XY" algorithm using random forests with traditional variable importance under permutation for testing conditional independence.
#' @param X Matrix of predictors.
#' @param Y The outcome variable.
#' @param T The treatment (exposure) variable. Must be dichotomous 0/1.
#' @param perms The number of permutation replications used to create a null distribtuion for importance values. The default value is \code{20}.
#' @param ntree The number of trees for the random forest run. The default value is \code{200}.
#' @param pctl The percentile of the null permutation distribution of importance used as a threshold for variable selection. The default is \code{pctl = .95}.
#' @param scale Should the permutation importances be ``scaled''? The default value is \code{FALSE}.
#' @keywords Random Forest Variable Selection Permutation Importance
#' @export

RFXY_trad <- function(X, Y, T,
                  perms = 20,
                  ntree = 200,
                  pctl = .95,
                  scale = FALSE) {
  if(is.null(X)) return(NULL)
  if(dim(X)[2] == 0) return(NULL)
  if(sum(is.na(X), is.na(Y), is.na(T)) > 0) stop("No missing values are allowed.")
  if(length(table(T)) != 2) stop("Exposure variable T must have exactly two categories.")
  T <- factor(T); levels(T) <- 0:1
  Xsubs0 <- X[T == 0, , drop = FALSE]; Xsubs1 <- X[T == 1, , drop = FALSE]
  Y0 <- Y[T == 0]; Y1 <- Y[T == 1]
  X0 <- RFVS_perm_trad(x = Xsubs0, y = Y0, ntree = ntree,
                   perms = perms, pctl = pctl,
                   scale = scale)
  X1 <- RFVS_perm_trad(x = Xsubs1, y = Y1, ntree = ntree,
                   perms = perms, pctl = pctl,
                   scale = scale)
  X0 <- rownames(X0)[which(X0[,1] > 0 & X0[,1] >= X0[,2])]
  X1 <- rownames(X1)[which(X1[,1] > 0 & X1[,1] >= X1[,2])]
  XY <- sort(union(X0, X1))
  if(is.null(XY)) return(NULL)
  if(length(XY) == 0) return(NULL)
  return(XY) }


