#' Wrapper for \code{rf_xy}.
#'
#' This function uses traditional random forest importance to iteratively implement rf_xy, paring down the covariate set until consecutive iterations are identical.
#' @param X Matrix of predictors.
#' @param Y The outcome variable.
#' @param T The treatment (exposure) variable. Must be dichotomous 0/1.
#' @param perms The number of permutation replications used to create a null distribtuion for importance values. The default is \code{20}.
#' @param ntree The number of trees for the random forest run. The default value is \code{200}.
#' @param pctl The percentile of the null distribution of importance used as a threshold for variable selection. The default is \code{pctl = .95}.
#' @param sidak Should a Sidak correction for multiple tests be made based on the number of predictors? The default is \code{TRUE}.
#' @param scale Should the permutation importances be ``scaled''? The default is \code{FALSE}.
#' @param iterate Should variable selection run iteratively until no new variables are dropped? The default is \code{FALSE}.
#' @keywords Random Forest Variable Selection Permutation Importance
#' @export

RFXY_trad_wrap <- function(X, Y, T,
                       perms = 20,
                       ntree = 200,
                       pctl = .95,
                       sidak = TRUE,
                       scale = FALSE,
                       iterate = FALSE) {
  if(iterate) {print("Iteration 1"); i <- 1}
  if(is.null(X)) return(NULL)
  if(dim(X)[2] == 0) return(NULL)
  if(sum(is.na(X), is.na(Y), is.na(T)) > 0) stop("No missing values are allowed.")
  if(sidak) { pctl <- (pctl)^(1/dim(X)[2]) }
  xy1 <- RFXY_trad(X = X, Y = Y, T = T, perms = perms,
                   ntree = ntree, pctl = pctl,
                   scale = scale)
  if(is.null(xy1)) return(NULL)
  if(!iterate) return(xy1)
  xy2 <- RFXY_trad(X = X[, xy1, drop = FALSE], Y = Y, T = T,
                   perms = perms, ntree = ntree, pctl = pctl,
                   scale = scale)
  if(is.null(xy2)) return(NULL)
  while(!identical(xy1, xy2)) { i <- i + 1; print(paste0("Iteration ", i))
  xy1 <- xy2
  xy2 <- RFXY_trad(X = X[, xy1, drop = FALSE], Y = Y, T = T,
                   perms = perms, ntree = ntree, pctl = pctl,
                   scale = scale) }
  if(is.null(xy2)) return(NULL)
  return(xy2)
}
