#' Permutation importance based on random forests
#'
#' This function uses package ranger to calculate traditional random forest permutation importance.
#' @param x Matrix of predictors.
#' @param y The outcome variable.
#' @param ntree Number of trees for the random forest run. The default is \code{200}.
#' @param scale Should the permutation importances be ``scaled''? The default is \code{FALSE}.
#' @param mtry Number of predictors sampled for the construction of each tree. The default is \code{max(floor(ncol(x)/3), 1)} for factor variables and \code{floor(sqrt(ncol(x)))} for non-factor variables.
#' @keywords Random Forest Variable Selection Permutation Importance
#' @export

RF_vimp <- function(x, y,
                    ntree = 200,
                    scale = FALSE,
                    mtry = if (!is.null(y) && !is.factor(y))
                      max(floor(ncol(x)/3), 1) else floor(sqrt(ncol(x)))) {
  if(is.null(x)) return(NULL)
  if(dim(x)[2] == 0) return(NULL)
  if(sum(is.na(x), is.na(y)) > 0) stop("No missing values are allowed.")
  rf <- ranger::ranger(formula = Y ~ ., data = data.frame(cbind(x,"Y" = y)),
                       num.trees = ntree,
                       mtry = mtry,
                       importance = "permutation",
                       scale.permutation.importance = scale)
  return(rf$variable.importance)
}
